package cz.upol.jj1.collections.lab07;

/**
 * LinkedList implementation for integers
 */
public class IntegerLinkedList {

    /**
     * Represents an item (one node) in IntegerLinkedList
     */
   protected class IntegerNode {
        // Vnorena trida (tzv. nested class nebo inner class).
        // Tato je protected, tedy s jejimi instancemi muzeme pracovat jen v ramci balicku (pres instanci nadrazene tridy)
        // a protoze jsme v kodu nadrazene tridy tak v inner class muzeme pracovat s vnitrnim stavem nadrazene instance.
        // Je mozne delat i public/private/default vnorene tridy.
        // Instance jsou z vnejsku dostupne pres OuterClass.InnerClass innerObject = outerObject.new InnerClass(); (omezeno modifikatory pristupu)
        // Pripadne staticke zanorene tridy dostupne jako staticky element nadrazene tridy.
        /**
         * Node value
         */
        private int value;

        /**
         * Pointer to the next node in the list.
         * If NULL this node is the last one.
         */
        private IntegerNode next;

        public IntegerNode(int value, IntegerNode next) {
            this.value = value;
            this.next = next;
        }

        public IntegerNode(int value) {
            this.value = value;
            this.next = null;
        }

        public IntegerNode getNext() {
            return next;
        }

        public int getValue() {
            return value;
        }

        //pokud potrebujete pristoupit k this v nadrazene tride pouzijte
        //LinkedList.this

        /**
         * Gets length of list which contains this node.
         * Weird, eh?
         * Better to keep it private.
         * @return Number of elemetns in the parrent list
         */
        private int ownersSize(){
            return IntegerLinkedList.this.size();
        }

        /**
         * Extremely useless example of
         * public static inner class of inner class
         */
        public static class NodesJoke {
            /**
             * Nodes be like...
             */
            public static String theJoke() {
                return "I know a guy who knows a guy ...";
            }
        }
    }

    /**
     * List's first node.
     * If NULL, then the list is empty
     */
    protected IntegerNode head;


    private int secret;
    public IntegerLinkedList() { }

    public IntegerLinkedList(int value) {
        insert(value);
    }

    /**
     * Gets head of the list.
     * @return the first value in the list or -1, if the list is empty.
     */
    public int head(){
        if (head == null) {
            return -1;
        }
        return head.value;
    }

    /**
     * Inserts a new value to the list
     * @param value Value to be inserted
     */
    public void insert(int value) {
        if (head == null) {
            head = new IntegerNode(value);
            return;
        }
        IntegerNode current = head;
        
        while (current.getNext() != null) {
            current = current.getNext();
        }
        
        current.next = new IntegerNode(value);
    }

    /**
     * Number of elements in the list.
     * @return number of elements
     **/
    public int size(){
        IntegerNode n  = this.head;
        int res = 0;
        while (n != null){
            res++;
            n = n.next;
        }
        return res;
    }


    @Override
    public String toString() {
        StringBuilder description = new StringBuilder();
        description.append("[");

        IntegerNode current = head;

        while (current.getNext() != null) {
            description.append(current.getValue());
            description.append(", ");
            current = current.getNext();
        }

        description.append(current.getValue());
        description.append("]");

        return description.toString();
    }

    /**
     * A bad riddle...
     * @return the answer
     */
    public String whatDoesANodeSay(){
        return IntegerNode.NodesJoke.theJoke();
    }
}
